// --------------------------------------------------------------------
//       B a d P k t . c p p
//                                                                     
//        Fido messages tracker                            
//        Work with bad packets.
// --------------------------------------------------------------------
//        Copyright (c) 1998,99 by Fyodor Ustinov                         
//                              FIDONet 2:5020/79                      
//        Copyright (c) 2004 by Alex Soukhotine
//                              FIDONet 2:5030/1157
//                                                                     
//        All rights reserved.                                         
// --------------------------------------------------------------------
#ifdef __MINGW32__
#ifndef _STAT_DEFINED
#define _STAT_DEFINED 1
#endif
#endif
#ifndef UNIX
#include <io.h>
#else
#define stricmp strcasecmp
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "constant.hpp"
#include "help.hpp"
#include "utils.hpp"
#include "vars.hpp"
#include "configure.hpp"
#include "scandir.hpp"
#include "ufmtypes.h"
#include "outbound.hpp"

// --------------------------------------------------------------------

int SetBadPktMode(tBadMsgMode bpmode, char *bpath) {
static int BadPktAlready = FALSE;

   if (BadPktAlready) {
      yyerror("BadPackets: already defined.");
      return(-1);
   }
   BadPktAlready = TRUE;
   BadPktMode = bpmode;
   if (bpath != NULL) {
   
    if (!DirExists(bpath)) {
      yyerror("Unable to open bad PKT directory.");
      return (-1);
    }
    
    BadPktDir = strdup(bpath);
    
   }

   return (0);
}

// --------------------------------------------------------------------

int BadPkt(char *PktName)
{
 char Buff[1024];
 Buff[0]='\0';
 if (BadPktMode == SKIP)
  Log.Level(LOGE) << "   PKT '" << PktName << "' skipped." << EOL; 
 else if (BadPktMode == MOVE)
 {
  strcpy(Buff,MakeAPktName(BadPktDir));
  if (FileMove(Buff,PktName))
   Log.Level(LOGE) << "   PKT '" << PktName << "' moved to '" << Buff << "'." << EOL;
  else
  {
   Log.Level(LOGE) << "   Can't move PKT '" << PktName << "' to '" << Buff << "'." << EOL;
   return (-1);
  }
 }
 else if (BadPktMode == REMOVE)
 {
  if (!unlink(PktName))
   Log.Level(LOGE) << "   PKT '" << PktName << "' removed." << EOL;
  else
  {
   Log.Level(LOGE) << "   Can't remove PKT '" << PktName << "'." << EOL;
   return (-1);
  }    
 }
 return (0);
}

// ---------------------------- END --------------------------------------
